/* $Header: /belch_a/users/rearl/tinymuck/src/RCS/property.c,v 1.5 90/09/16 04:42:47 rearl Exp $ */

/*
 * $Log:	property.c,v $
 * Revision 1.5  90/09/16  04:42:47  rearl
 * Preparation code added for disk-based MUCK.
 * 
 * Revision 1.4  90/09/15  22:27:45  rearl
 * Fixed dangerous COMPRESS-related bug.
 * 
 * Revision 1.3  90/09/10  02:19:49  rearl
 * Introduced string compression of properties, for the
 * COMPRESS compiler option.
 * 
 * Revision 1.2  90/08/02  18:50:00  rearl
 * Fixed NULL pointer problem in get_property_class().
 * 
 * Revision 1.1  90/07/19  23:04:03  casie
 * Initial revision
 * 
 *
 */
 
#include "copyright.h"
#include "config.h"

#include "db.h"
#include "externs.h"

#ifdef COMPRESS
extern const char *compress(const char *);
#define alloc_compressed(x) alloc_string(compress(x))
#else /* !COMPRESS */
#define alloc_compressed(x) alloc_string(x)
#endif /* COMPRESS */

/* property.c
   A whole new lachesis mod.
   Adds property manipulation routines to TinyMUCK.   */

/* IN ALL CASES, the argument "value" is referred to only when "class" is
   a NULL string.                                                       */

/* adds a new property to an object */
void
add_property(dbref player, const char *type, const char *class, int value)
{
  struct plist  *p, *l;

  if (DBFETCH(player)->properties)
    {
      /* stick properties at the end because most frequently used ones
	 will be put in front.                                      */
      for (l = DBFETCH(player)->properties; l; l = l -> next)
	{
	  if (!string_compare(type, l -> type))
	    {
	      if (class)
		{
		  if (l -> class)
		    free((void *) l -> class);
		  l -> class = alloc_compressed(class);
		}
	      else
		l -> value = value;
	      return;
	    }
	  p = l;
	}
      p -> next = new_prop();
      p = p -> next;
      p -> type = alloc_string(type);
      if (class)
	p -> class = alloc_compressed(class);
      else
	p -> value = value;
    }
  else
    {
      /* no current property list --- create property list */
      p = new_prop();
      p -> type = alloc_string(type);
      if (class)
	p -> class = alloc_compressed(class);
      else
	p -> value = value;
      DBSTORE(player, properties, p);
    }
}

/* removes property list --- if it's not there then ignore */
void
remove_property(dbref player, const char *type)
{
  struct plist *last, *curr;

  for (last = curr = DBFETCH(player)->properties; curr; curr = curr -> next)
    {
      if (!string_compare(curr -> type, type))
	{
	  if (DBFETCH(player)->properties == curr)
	    {
	      DBSTORE(player, properties, curr -> next);
	    }
	  else
	    last -> next = curr -> next;
	  free_prop(curr);
	  return;
	}
      last = curr;
    }
}

/* checks if object has property, returning 1 if it or any of it's contents has
   the property stated                                                      */
int
has_property(dbref player, const char *type, const char *class, int value)
{
  dbref things;

  if (has_property_strict(player, type, class, value))
    return 1;

  for (things = DBFETCH(player)->contents; things != NOTHING; things = DBFETCH(things)->next)
    if (has_property_strict(things, type, class, value))
      return 1;

  return 0;
}

/* checks if object has property, returning 1 if it has the property asked for */
int
has_property_strict(dbref player, const char *type, const char *class, int value)
{
  struct plist *p;
  const char *str;

#ifdef COMPRESS
  str = compress(DoNull(class));
#else /* !COMPRESS */
  str = DoNull(class);
#endif /* COMPRESS */
  
  for (p = DBFETCH(player)->properties; p; p = p -> next)
    if (!string_compare(type, p -> type))
      {
	if (p -> class)
	  return (!string_compare(str, p -> class));
	else
	  return (value == p -> value);
      }

  return 0;
}

/* return class of property */
const char *
get_property_class(dbref player, const char *type)
{
  struct plist *p;

  for (p = DBFETCH(player)->properties; p; p = p -> next)
    if (!string_compare(type, p -> type))
      return(p -> class);

  return (const char *) 0;
}

/* return value of property */
int
get_property_value(dbref player, const char *type)
{
  struct plist *p;

  for (p = DBFETCH(player)->properties; p; p = p -> next)
    if (!string_compare(type, p -> type))
      return p -> value;

  return 0; /* I guess this is as good as any other.. */
}

/* copies properties */
struct plist *
copy_prop(dbref old)
{
  struct plist *p;
  struct plist *new = 0, *first = 0;

  for (p = DBFETCH(old)->properties; p; p = p -> next)
    {
      if (!first)	first = new = new_prop();
      else              new -> next = new_prop();

      if (new -> next)
	new = new -> next;
      new -> type = alloc_string(p -> type);
      new -> class = alloc_compressed(p -> class);
      new -> value = p -> value;
    }
  return first;
}

/* return old gender values for pronoun substitution code */
int
genderof(dbref player)
{
  if (has_property_strict(player, "sex", "male", 0))
    return GENDER_MALE;
  else if (has_property_strict(player, "sex", "female", 0))
    return GENDER_FEMALE;
  else if (has_property_strict(player, "sex", "neuter", 0))
    return GENDER_NEUTER;
  else
    return GENDER_UNASSIGNED;
}
